// @ts-check
import path from "path";
import fs from "fs";
import grabHostNames from "../utils/grab-host-names";
import apiPost from "../functions/api/query/post";
import debugLog from "../utils/logging/debug-log";
/**
 * # Make a post request to Datasquirel API
 */
export default async function post({ key, query, queryValues, database, tableName, user_id, forceLocal, debug, }) {
    const grabedHostNames = grabHostNames();
    const { host, port, scheme } = grabedHostNames;
    if (debug) {
        debugLog({
            log: grabedHostNames,
            addTime: true,
            label: "grabedHostNames",
        });
    }
    /**
     * Check for local DB settings
     *
     * @description Look for local db settings in `.env` file and by pass the http request if available
     */
    const { DSQL_DB_HOST, DSQL_DB_USERNAME, DSQL_DB_PASSWORD, DSQL_DB_NAME } = process.env;
    if ((DSQL_DB_HOST === null || DSQL_DB_HOST === void 0 ? void 0 : DSQL_DB_HOST.match(/./)) &&
        (DSQL_DB_USERNAME === null || DSQL_DB_USERNAME === void 0 ? void 0 : DSQL_DB_USERNAME.match(/./)) &&
        (DSQL_DB_PASSWORD === null || DSQL_DB_PASSWORD === void 0 ? void 0 : DSQL_DB_PASSWORD.match(/./)) &&
        (DSQL_DB_NAME === null || DSQL_DB_NAME === void 0 ? void 0 : DSQL_DB_NAME.match(/./)) &&
        global.DSQL_USE_LOCAL) {
        /** @type {import("../types").DSQL_DatabaseSchemaType | undefined} */
        let dbSchema;
        try {
            const localDbSchemaPath = path.resolve(process.cwd(), "dsql.schema.json");
            dbSchema = JSON.parse(fs.readFileSync(localDbSchemaPath, "utf8"));
        }
        catch (error) { }
        if (debug) {
            debugLog({
                log: "Using Local DB ...",
                addTime: true,
            });
        }
        return await apiPost({
            dbFullName: database || DSQL_DB_NAME,
            query,
            dbSchema,
            queryValues,
            tableName,
            forceLocal,
            debug,
        });
    }
    /**
     * Make https request
     *
     * @description make a request to datasquirel.com
     */
    const httpResponse = await new Promise((resolve, reject) => {
        var _a;
        const reqPayloadString = JSON.stringify({
            query,
            queryValues,
            database: process.env.DSQL_API_DB_NAME || database,
            tableName: tableName ? tableName : null,
        }).replace(/\n|\r|\n\r/gm, "");
        try {
            JSON.parse(reqPayloadString);
        }
        catch (error) {
            (_a = global.ERROR_CALLBACK) === null || _a === void 0 ? void 0 : _a.call(global, `Error Parsing HTTP response for post action`, error);
            return {
                success: false,
                payload: null,
                error: "Query object is invalid. Please Check query data values",
            };
        }
        const reqPayload = reqPayloadString;
        const requPath = `/api/query/${user_id || grabedHostNames.user_id}/post`;
        const httpsRequest = scheme.request({
            method: "POST",
            headers: {
                "Content-Type": "application/json",
                "Content-Length": Buffer.from(reqPayload).length,
                Authorization: key ||
                    process.env.DSQL_FULL_ACCESS_API_KEY ||
                    process.env.DSQL_API_KEY,
            },
            port,
            hostname: host,
            path: requPath,
        }, 
        /**
         * Callback Function
         *
         * @description https request callback
         */
        (response) => {
            var str = "";
            response.on("data", function (chunk) {
                str += chunk;
            });
            response.on("end", function () {
                try {
                    resolve(JSON.parse(str));
                }
                catch (error) {
                    console.log("Route ERROR:", error.message);
                    resolve({
                        success: false,
                        payload: null,
                        error: error.message,
                        errPayload: str,
                    });
                }
            });
            response.on("error", (err) => {
                resolve({
                    success: false,
                    payload: null,
                    error: err.message,
                });
            });
        });
        httpsRequest.write(reqPayload);
        httpsRequest.on("error", (error) => {
            console.log("HTTPS request ERROR =>", error);
        });
        httpsRequest.end();
    });
    return httpResponse;
}
